@@ -245,7 +245,7 @@ is_within_constraint_region (MetaWaylandPointerConstraint *constraint,
   return is_within;
 }
 
-void
+static void
 meta_wayland_pointer_constraint_maybe_enable (MetaWaylandPointerConstraint *constraint)
 {
   MetaWaylandSeat *seat = constraint->seat;
@@ -336,12 +336,6 @@ meta_wayland_pointer_constraint_calculate_effective_region (MetaWaylandPointerCo
   return region;
 }
 
-cairo_region_t *
-meta_wayland_pointer_constraint_get_region (MetaWaylandPointerConstraint *constraint)
-{
-  return constraint->region;
-}
-
 MetaWaylandSurface *
 meta_wayland_pointer_constraint_get_surface (MetaWaylandPointerConstraint *constraint)
 {
@@ -610,7 +604,7 @@ locked_pointer_set_cursor_position_hint (struct wl_client   *client,
 
   /* Ignore a set cursor hint that was already sent after the constraint
    * was cancelled. */
-  if (!constraint->resource || constraint->resource != resource)
+  if (!constraint || !constraint->resource || constraint->resource != resource)
     return;
 
   constraint->hint_set = TRUE;
@@ -628,6 +622,9 @@ locked_pointer_set_region (struct wl_client   *client,
   MetaWaylandRegion *region =
     region_resource ? wl_resource_get_user_data (region_resource) : NULL;
 
+  if (!constraint)
+    return;
+
   meta_wayland_pointer_constraint_set_pending_region (constraint, region);
 }
 
@@ -741,6 +738,9 @@ confined_pointer_set_region (struct wl_client   *client,
   MetaWaylandRegion *region =
     region_resource ? wl_resource_get_user_data (region_resource) : NULL;
 
+  if (!constraint)
+    return;
+
   meta_wayland_pointer_constraint_set_pending_region (constraint, region);
 }
 
