@@ -36,20 +36,14 @@ G_DECLARE_FINAL_TYPE (MetaWaylandPointerConstraint,
                       META, WAYLAND_POINTER_CONSTRAINT,
                       GObject);
 
-typedef struct _MetaWaylandPointerConstraint MetaWaylandPointerConstraint;
-
 void meta_wayland_pointer_constraints_init (MetaWaylandCompositor *compositor);
 
-void meta_wayland_pointer_constraint_maybe_enable (MetaWaylandPointerConstraint *constraint);
-
 void meta_wayland_pointer_constraint_destroy (MetaWaylandPointerConstraint *constraint);
 
 MetaWaylandSeat * meta_wayland_pointer_constraint_get_seat (MetaWaylandPointerConstraint *constraint);
 
 cairo_region_t * meta_wayland_pointer_constraint_calculate_effective_region (MetaWaylandPointerConstraint *constraint);
 
-cairo_region_t * meta_wayland_pointer_constraint_get_region (MetaWaylandPointerConstraint *constraint);
-
 MetaWaylandSurface * meta_wayland_pointer_constraint_get_surface (MetaWaylandPointerConstraint *constraint);
 
 void meta_wayland_pointer_constraint_maybe_remove_for_seat (MetaWaylandSeat *seat,
